<?php
/**
 * Класс базы данных.
 * Возвращает идентификатор соединения
 *
 * @copyright 
 * @link 
 * @package Majestic
 * @subpackage DB
 * @since 
 * @version SVN: $Id$
 * @filesource $URL$
 */
class DBConnector
{
    private $handlers = array();
    static private $instance = null;
    static public $queries   = array();
    
    /**
     * Запрещаем new и клонирование
     */
    private function __construct(){}
    
    private function __clone(){}
    
    /**
     * Единственный способ инициализации. Singletone как-никак.
     * @return DBConnector
     */
    static public function getInstance()
    {
        if (self::$instance == null) {
            self::$instance = new DBConnector();
        }
        return self::$instance;
    }

    /**
     * Получение соединения.
     * Если соединение с такими параметрами уже есть - новое не создается.
     *
     * @param array $db_settings - массив настроек
     * @return resource - идентификатор соединения
     */
    public function getConnect($db_settings)
    {
        $handler_name = $this->getConnectionName($db_settings);

        if (isset($this->handlers[$handler_name])) {
            return $this->handlers[$handler_name];
        }

        if (!$handler = mysqli_connect($db_settings['host'], $db_settings['user'], $db_settings['password'], $db_settings['database'])) {
            throw new MJException('Can\'t connect to DB '.mysqli_connect_error(), 2);
        }

        mysqli_query($handler, "SET NAMES 'utf8'"); //cheat!!!
        return $this->handlers[$handler_name] = $handler;
    }

    protected function getConnectionName($db_settings)
    {
        return $db_settings['host'] . '-' . $db_settings['database'];
    }
}
?>